new;
library optmum,pgraph;

#include regGMM.src

X0 = 10;
mu = 0.2;
sigma = 0.5;
h = 0.1;
Nobs = 500;
sv = mu|sigma;

/* Generate a geometric Brownian motion */

t = seqa(0,h,Nobs);
xt = x0*exp(   (mu-0.5*sigma^2)*t
               + sigma*cumsumc(0|rndn(Nobs-1,1)*sqrt(h))
          );

proc ml(theta);
  local e,mu,sigma,epsilon,logl;

  e = ln(xt[2:Nobs]./xt[1:Nobs-1]);

  mu = theta[1];
  sigma = theta[2];

  epsilon = e - (mu-0.5*sigma^2)*h;

  logl = -0.5*ln(2*pi) - 0.5*ln(sigma^2*h) - 0.5*epsilon^2/(sigma^2*h);

  retp(logl);
endp;

fn negLogL(theta) = -sumc(ml(theta));

proc gmm(theta);
  local e,mu,sigma,epsilon,M;

  e = ln(xt[2:Nobs]./xt[1:Nobs-1]);

  mu = theta[1];
  sigma = theta[2];

  epsilon = e - (mu-0.5*sigma^2)*h;

  M = epsilon~(epsilon^2-(sigma^2*h));

  retp(M);
endp;

parnm = "mu"|"sigma";

{theta1,fmin,grd,retcode} = optmum(&negLogL,sv);

{theta2,stderr,Mcov,Qmin} = regGMM(&gmm,sv,0,0);

print; print;
print "                 True values        TDML            GMM";
call printfmt(parnm~sv~theta1~theta2,0~1~1~1);

